#!/usr/bin/python3
# -*- coding: utf-8 -*

# Programme de Photomaton avec possibilité de 1 photo ou 1 vidéo en slow motion
# Script TRES fortement basé sur un travail trouvé sur Internet et appelé
# boothy - Photobooth application for Raspberry Pi
# developed by Kenneth Centurion
#
# Cette version a été réalisée par Alban TREVILLY en juillet et août 2019
# Travail personnel effectué par plaisir et pour rendre service à Julien
#
# Cette application :
# - Prendra soit 1 photo, soit 1 vidéo avec un délai de 5 secondes (adaptez "PHOTO_DELAY" pour le délai).
# - Superposera un texte à l'écran avec un compte à rebours.

import picamera
import os
import time
import RPi.GPIO as GPIO
from PIL import Image, ImageDraw, ImageFont
import pygame
from pygame.locals import *

LARGEUR_ECRAN     = 1280
HAUTEUR_ECRAN    = 960
LARGEUR_PHOTO      = 1440
HAUTEUR_PHOTO     = 1080
LARGEUR_APERCU  = 1280
HAUTEUR_APERCU  = 960
PHOTO_DELAY      = 5 #délai en secondes avant prise de la photo
overlay_renderer = None
buttonEvent      = False
buttonEvent2      = False
buttonEvent3      = False

pygame.init()
#screen = pygame.display.set_mode((0,0),pygame.FULLSCREEN)
screen = pygame.display.set_mode((1920,1080),RESIZABLE)
width, height = screen.get_size()


#on initialise les GPIO en écoute
#Attention au choix des ports ; référez-vous au site https://fr.pinout.xyz/
#J'ai pris les BCM18(PIO12) et BCM26(GPIO37) car les masses sont à côté
GPIO.setmode(GPIO.BCM)
GPIO.setup(18, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(26, GPIO.IN, pull_up_down=GPIO.PUD_UP)

    
def cleanUp():
    GPIO.cleanup()


def countdownFrom(secondsStr):
    secondsNum = int(secondsStr)
    if secondsNum >= 0 :
        while secondsNum > 0 :
            addPreviewOverlay(200,100,240,str(secondsNum))
            time.sleep(1)
            secondsNum=secondsNum-1

def captureImage(imageName):
    addPreviewOverlay(50,100,100,"Souriez !")
    #Sauvegarde de l'image
    camera.capture(imageName, resize=(LARGEUR_PHOTO, HAUTEUR_PHOTO))
    

def captureVideo(imageNameVideo):
    addPreviewOverlay(50,80,80,"Souriez, vous êtes filmé(e)(s) !")
    #Sauvegarde de l'image
    camera.start_recording(imageNameVideo)
    camera.wait_recording(5)
    camera.stop_recording()



def addPreviewOverlay(xcoord,ycoord,fontSize,overlayText):
    global overlay_renderer
    img = Image.new("RGB", (LARGEUR_APERCU, HAUTEUR_APERCU))
    draw = ImageDraw.Draw(img)
    draw.font = ImageFont.truetype(
                    "/home/pi/Photomaton_PV/Retrofunk_Script_Personal_Use.otf",fontSize)
    draw.text((xcoord,ycoord), overlayText, (255, 20, 147))

    if not overlay_renderer:
        overlay_renderer = camera.add_overlay(img.tobytes(),
                                              layer=3,
                                              size=img.size,
                                              alpha=128);
    else:
        overlay_renderer.update(img.tobytes())

def AfficherPhoto(fileName): # affiche NomPhoto
    print("loading image: " + fileName)
    background = pygame.image.load(fileName);
    background.convert_alpha()
    background = pygame.transform.scale(background,(LARGEUR_PHOTO,HAUTEUR_PHOTO))
    screen.blit(background,(240,0),(0,0,LARGEUR_PHOTO,HAUTEUR_PHOTO))
    pygame.display.flip()


def Photo():
    print("Démarrage de la séquence '1 photo'")

    fileName = "/var/www/html/Photos/"+time.strftime("%Y%m%d-%H%M%S")+".jpg"
    fileNameCrop = "/var/www/html/Photos/"+time.strftime("%Y%m%d-%H%M%S")+"_crop.jpg"
    fileName_tampon = "/var/www/html/Photos/"+time.strftime("%Y%m%d-%H%M%S")+"_tampon.jpg"

    print("Nom de fichier créé : "+fileName)

    countdownFrom(PHOTO_DELAY)
    captureImage(fileName)
    time.sleep(1)
    addPreviewOverlay(50,100,100,"Merci !")

    os.system("convert -flop "+fileName+" "+fileNameCrop+"")
    os.system("convert "+fileNameCrop+" /home/pi/Photomaton_PV/tampon.png -gravity SouthEast -composite "+fileName_tampon+"")
    AfficherPhoto(fileName_tampon)
    camera.stop_preview()
    time.sleep(5)
    camera.start_preview()
    os.system("rm "+fileName+"")
    os.system("rm "+fileNameCrop+"")

    
    

def Video():
    print("Démarrage de la séquence 'video'")

    fileNameVideo = "/var/www/html/Videos/"+time.strftime("%Y%m%d-%H%M%S")+".h264"
    print("Nom de fichier créé : "+fileNameVideo)

    countdownFrom(PHOTO_DELAY)
    captureVideo(fileNameVideo)
    addPreviewOverlay(50,100,90,"Encodage vidéo ; patience !")
    time.sleep(3)
    os.system("ffmpeg -r 10 -i /var/www/html/Videos/*.h264 -vcodec copy /var/www/html/Videos/"+time.strftime("%Y%m%d-%H%M%S")+"_non_Flip.mp4")
    os.system("sudo rm /var/www/html/Videos/*.h264")
    


    
def initCamera(camera):
    print("Initialisation de la caméra.")
    #camera settings
    camera.resolution            = (LARGEUR_PHOTO,HAUTEUR_PHOTO)
    camera.framerate             = 24
    camera.sharpness             = 0
    camera.contrast              = 0
    camera.brightness            = 50
    camera.saturation            = 0
    camera.ISO                   = 0
    camera.video_stabilization   = False
    camera.exposure_compensation = 0
    camera.exposure_mode         = 'auto'
    camera.meter_mode            = 'average'
    camera.awb_mode              = 'auto'
    camera.image_effect          = 'none'
    camera.color_effects         = None
    camera.rotation              = 0
    camera.hflip                 = True
    camera.vflip                 = False
    camera.crop                  = (0.0, 0.0, 0.0, 0.0)
    camera.exif_tags['IFD0.Artist'] = 'Photomaton de Julien par Alban'
    camera.exif_tags['IFD0.Copyright'] = 'Copyright (c) 2019 Julien!'


def onButtonPressBis():
    print("Demande faite pour 1 photo !")
    Photo()
    #reset the initial welcome message
    addPreviewOverlay(20,100,55,"Prenez position ... et ...")

def onButtonPressTer():
    print("Demande faite pour 1 vidéo !")
    Video()
    #reset the initial welcome message
    addPreviewOverlay(20,100,55,"Prenez position ... et ...")



def onButtonDePressBis():
    print("Bouton relâché !")

def onButtonDePressTer():
    print("Bouton relâché !")


#Flux initial
with picamera.PiCamera() as camera:
    camera.framerate = 90

    try:
        initCamera(camera)
        print("Démarrage de l'aperçu")
        camera.start_preview()
        ###camera.annotate_text = "Texte possible ici si pas de tampon et pas de retournement"
        addPreviewOverlay(20,100,55,"Prenez position ... et ...")
    

        print("Démarrage de la boucle du programme")
        while True:
            input_state2 = GPIO.input(26)
            input_state3= GPIO.input(18)

            if input_state2 == False : 
                if buttonEvent2 == False :
                    buttonEvent2 = True
                    onButtonPressBis()
            else :
                if buttonEvent2 == True :
                    buttonEvent2 = False
                    onButtonDePressBis()
      
            if input_state3 == False : 
                if buttonEvent3 == False :
                    buttonEvent3 = True
                    onButtonPressTer()
            else :
                if buttonEvent3 == True :
                    buttonEvent3 = False
                    onButtonDePressTer()
                    
    except BaseException:
        print("Heu ... Exception non gérée : ")
        camera.close()
        cleanUp()
    finally:
        print("Au revoir ...")
        cleanUp()
        camera.close()

#end
